'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT16 implements DPT {
  id = '16';
  name = '14-character string';
  bufferLength = 14;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): string {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);


    return buffer.toString('ascii');
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: string): Buffer {
    if (typeof value !== 'string')
      throw new InvalidValueError('Must supply a string value')

    const buf = Buffer.alloc(14);
    buf.write(value, 'ascii');
    return buf;
  }


  subtypes: {
    // 16.000 ASCII string
    '000': {
      use: 'G',
      name: 'DPT_String_ASCII',
      desc: 'ASCII string',
      force_encoding: 'US-ASCII',
    },

    // 16.001 ISO-8859-1 string
    '001': {
      use: 'G',
      name: 'DPT_String_8859_1',
      desc: 'ISO-8859-1 string',
      force_encoding: 'ISO-8859-1',
    },
  };

}
