import { expect } from "chai";

import { DPT7 } from "../src/DPT7"
import { BufferLengthError } from "../src/errors/BufferLengthError";
import { InvalidValueError } from "../src/errors/InvalidValueError";

import { compareBuffers } from "./util"

let bufferA = Buffer.from([1, 0])
let bufferMax = Buffer.from([255, 255])

let bufferEmpty = Buffer.from([])
let bufferBiggerSize = Buffer.from([1, 0, 2, 4])

describe("Test DPT007", (): void => {

  let dpt = new DPT7();

  it("Decode buffer common", async function () {
    const value = dpt.decoder(bufferA)
    expect(value).is.equal(256);
  });

  it("Decode buffer max", async function () {
    const value = dpt.decoder(bufferMax)
    expect(value).is.equal(65535);
  });

  it("Decode empty buffer", async function () {
    var testFunction = function () {
      const value = dpt.decoder(bufferEmpty)
    }
    expect(testFunction).to.throw(BufferLengthError);
  });

  it("Decode oversized buffer", async function () {
    var testFunction = function () {
      const value = dpt.decoder(bufferBiggerSize)
    }
    expect(testFunction).to.throw(BufferLengthError);
  });

  /* Encoder tests */
  it("encode valid", async function () {
    expect(compareBuffers(dpt.encoder(256), bufferA)).is.true;
    expect(compareBuffers(dpt.encoder(65535), bufferMax)).is.true;
  });

  it("encode invalid", async function () {
    var testFunction = function () {
      const value = dpt.encoder(-200)
    }

    expect(testFunction).to.throw(InvalidValueError);
  });

  it("encode undefined", async function () {
    var testFunction = function () {
      const value = dpt.encoder(undefined)
    }
    expect(testFunction).to.throw(InvalidValueError);
  });
});
