import { expect } from "chai";

import { DPT13 } from "../src/DPT13"
import { BufferLengthError } from "../src/errors/BufferLengthError";
import { InvalidValueError } from "../src/errors/InvalidValueError";

import { compareBuffers } from "./util"

describe("Test DPT013", (): void => {

  let dpt = new DPT13();

  it("Decode buffer acceptable", async function () {
    let value = 0x24101011
    let buffer = Buffer.from([0x24, 0x10, 0x10, 0x11])
    const decoded = dpt.decoder(buffer)
    expect(decoded).is.equal(value);
  });

  it("Decode empty buffer", async function () {
    let bufferEmpty = Buffer.from([])
    var testFunction = function () {
      const value = dpt.decoder(bufferEmpty)
    }
    expect(testFunction).to.throw(BufferLengthError);
  });

  it("Decode oversized buffer", async function () {
    let bufferBiggerSize = Buffer.from([0x20, 0x15, 0, 0, 0])

    var testFunction = function () {
      const value = dpt.decoder(bufferBiggerSize)
    }
    expect(testFunction).to.throw(BufferLengthError);
  });

  /* Encoder tests */
  it("encode valid", async function () {
    let value = 0x00f040cc
    let buffer = Buffer.from([0x00, 0xf0, 0x40, 0xcc])

    expect(compareBuffers(dpt.encoder(value), buffer)).is.true;
  });

  it("encode not integer", async function () {
    var testFunction = function () {
      const value = dpt.encoder(Math.PI)
    }
    expect(testFunction).to.throw(InvalidValueError);
  });

  it("encode out of range", async function () {
    var testFunction = function () {
      const value = dpt.encoder(-Math.pow(2, 31) - 1)
    }
    expect(testFunction).to.throw(InvalidValueError);
  });

  it("encode undefined", async function () {
    var testFunction = function () {
      const value = dpt.encoder(undefined)
    }
    expect(testFunction).to.throw(InvalidValueError);
  });
});