# Introduction

This folder has the dpt library written in TS.
Original code is in the knx.js library, but it's not exposed.

From [here](https://www.promotic.eu/en/pmdoc/Subsystems/Comm/PmDrivers/KNXDTypes.htm):

| Identifier | Name                                     | Size    | PROMOTIC data type | Interpretation range                                                                                                                                                                                           | Note     |
| ---------- | ---------------------------------------- | ------- | ------------------ | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | -------- |
| 1.001      | DPT_Switch                               | 1 bit   | Boolean            | 0 = off, 1 = on                                                                                                                                                                                                |          |
| 1.002      | DPT_Bool                                 | 1 bit   | Boolean            | 0 = false, 1 = true                                                                                                                                                                                            |          |
| 1.003      | DPT_Enable                               | 1 bit   | Boolean            | 0 = disable, 1 = enable                                                                                                                                                                                        |          |
| 1.004      | DPT_Ramp                                 | 1 bit   | Boolean            | 0 = no ramp, 1 = ramp                                                                                                                                                                                          |          |
| 1.005      | DPT_Alarm                                | 1 bit   | Boolean            | 0 = no alarm, 1 = alarm                                                                                                                                                                                        |          |
| 1.006      | DPT_BinaryValue                          | 1 bit   | Boolean            | 0 = low, 1 = high                                                                                                                                                                                              |          |
| 1.007      | DPT_Step                                 | 1 bit   | Boolean            | 0 = decrease, 1 = increase                                                                                                                                                                                     |          |
| 1.008      | DPT_UpDown                               | 1 bit   | Boolean            | 0 = up, 1 = down                                                                                                                                                                                               |          |
| 1.009      | DPT_OpenClose                            | 1 bit   | Boolean            | 0 = open, 1 = close                                                                                                                                                                                            |          |
| 1.010      | DPT_Start                                | 1 bit   | Boolean            | 0 = stop, 1 = start                                                                                                                                                                                            |          |
| 1.011      | DPT_State                                | 1 bit   | Boolean            | 0 = inactive, 1 = active                                                                                                                                                                                       |          |
| 1.012      | DPT_Invert                               | 1 bit   | Boolean            | 0 = not inverted, 1 = inverted                                                                                                                                                                                 |          |
| 1.013      | DPT_DimSendStyle                         | 1 bit   | Boolean            | 0 = start-stop, 1 = cyclically                                                                                                                                                                                 |          |
| 1.014      | DPT_InputSource                          | 1 bit   | Boolean            | 0 = fixed, 1 = calculated                                                                                                                                                                                      |          |
| 1.015      | DPT_Reset                                | 1 bit   | Boolean            | 0 = dummy, 1 = trigger                                                                                                                                                                                         |          |
| 1.016      | DPT_Ack                                  | 1 bit   | Boolean            | 0 = dummy, 1 = trigger                                                                                                                                                                                         |          |
| 1.017      | DPT_Trigger                              | 1 bit   | Boolean            | 0,1 = trigger                                                                                                                                                                                                  |          |
| 1.018      | DPT_Occupancy                            | 1 bit   | Boolean            | 0 = not occupied, 1 = occupied                                                                                                                                                                                 |          |
| 1.019      | DPT\_Window\_Door                        | 1 bit   | Boolean            | 0 = closed, 1 = open                                                                                                                                                                                           |          |
| 1.021      | DPT_LogicalFunction                      | 1 bit   | Boolean            | 0 = OR, 1 = AND                                                                                                                                                                                                |          |
| 1.022      | DPT\_Scene\_AB                           | 1 bit   | Boolean            | 0 = sceneA, 1 = sceneB                                                                                                                                                                                         |          |
| 1.023      | DPT\_ShutterBlinds\_Mode                 | 1 bit   | Boolean            | 0 = UpDown only, 1 = UpDown + StepStop                                                                                                                                                                         |          |
| 1.100      | DPT_Heat/Cool                            | 1 bit   | Boolean            | 0 = cooling, 1 = heating                                                                                                                                                                                       |          |
| 2.001      | DPT\_Switch\_Control                     | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.002      | DPT\_Bool\_Control                       | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.003      | DPT\_Enable\_Control                     | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.004      | DPT\_Ramp\_Control                       | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.005      | DPT\_Alarm\_Control                      | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.006      | DPT\_BinaryValue\_Control                | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.007      | DPT\_Step\_Control                       | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.008      | DPT\_Direction1\_Control                 | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.009      | DPT\_Direction2\_Control                 | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.010      | DPT\_Start\_Control                      | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.011      | DPT\_State\_Control                      | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 2.012      | DPT\_Invert\_Control                     | 2 bits  | Long               | C₁V₁<br><br>C = {0 (no control), 1 (control)}; V = {0,1} according to type 1.XXX                                                                                                                               |          |
| 3.007      | DPT\_Control\_Dimming                    | 4 bits  | Long               | C₁S₃<br><br>C = {0 (decrease), 1 (increase)}; S = \[000..111\]                                                                                                                                                 |          |
| 3.008      | DPT\_Control\_Blinds                     | 4 bits  | Long               | C₁S₃<br><br>C = {0 (up), 1 (down)}; S = \[000..111\]                                                                                                                                                           |          |
| 4.001      | DPT\_Char\_ASCII                         | 1 byte  | Long               | ASCII character                                                                                                                                                                                                |          |
| 4.002      | DPT\_Char\_8859_1                        | 1 byte  | Long               | ISO_8859 character                                                                                                                                                                                             |          |
| 5.001      | DPT_Scaling                              | 1 byte  | Long               | \[0 … 100\]%                                                                                                                                                                                                   |          |
| 5.003      | DPT_Angle                                | 1 byte  | Long               | \[0 … 360\]°                                                                                                                                                                                                   |          |
| 5.004      | DPT\_Percent\_U8                         | 1 byte  | Long               | \[0 … 255\]%                                                                                                                                                                                                   |          |
| 5.005      | DPT_DecimalFactor                        | 1 byte  | Long               |                                                                                                                                                                                                                |          |
| 5.006      | DPT_Tariff                               | 1 byte  | Long               | \[0 … 254\]                                                                                                                                                                                                    |          |
| 5.010      | DPT\_Value\_1_Ucount                     | 1 byte  | Long               | \[0 … 255\]                                                                                                                                                                                                    |          |
| 6.001      | DPT\_Percent\_V8                         | 1 byte  | Long               | \[-128 … 127\]%                                                                                                                                                                                                |          |
| 6.010      | DPT\_Value\_1_Count                      | 1 byte  | Long               | \[-128 … 127\]                                                                                                                                                                                                 |          |
| 6.020      | DPT\_Status\_Mode3                       | 1 byte  | Long               | A₁B₁C₁D₁E₁F₃<br><br>A,B,C,D,E = {0 (set), 1 (clear)}; F = {001 (mode 0), 010 (mode 1), 100 (mode 2)}                                                                                                           |          |
| 7.001      | DPT\_Value\_2_Ucount                     | 2 bytes | Long               | \[0 … 65535\] pulses                                                                                                                                                                                           |          |
| 7.002      | DPT_TimePeriodMsec                       | 2 bytes | Long               | \[0 … 65535\] ms                                                                                                                                                                                               |          |
| 7.003      | DPT_TimePeriod10Msec                     | 2 bytes | Long               | \[0 … 6553,5\] s                                                                                                                                                                                               | k=0,1    |
| 7.004      | DPT_TimePeriod100Msec                    | 2 bytes | Long               | \[0 … 655,35\] s                                                                                                                                                                                               | k=0,01   |
| 7.005      | DPT_TimePeriodSec                        | 2 bytes | Long               | \[0 … 65535\] s                                                                                                                                                                                                |          |
| 7.006      | DPT_TimePeriodMin                        | 2 bytes | Long               | \[0 … 65535\] min                                                                                                                                                                                              |          |
| 7.007      | DPT_TimePeriodHrs                        | 2 bytes | Long               | \[0 … 65535\] h                                                                                                                                                                                                |          |
| 7.010      | DPT_PropDataType                         | 2 bytes | Long               | \[0 … 65535\]                                                                                                                                                                                                  |          |
| 7.011      | DPT\_Length\_mm                          | 2 bytes | Long               | \[0 … 65535\] mm                                                                                                                                                                                               |          |
| 7.012      | DPT_UElCurrentmA                         | 2 bytes | Long               | \[0 … 65535\] mA                                                                                                                                                                                               |          |
| 7.013      | DPT_Brightness                           | 2 bytes | Long               | \[0..65535\] lux                                                                                                                                                                                               |          |
| 8.001      | DPT\_Value\_2_Count                      | 2 bytes | Long               | \[-32 768 … 32 767\]                                                                                                                                                                                           |          |
| 8.002      | DPT_DeltaTimeMsec                        | 2 bytes | Long               | \[-32 768 … 32 767\] ms                                                                                                                                                                                        |          |
| 8.003      | DPT_DeltaTime10Msec                      | 2 bytes | Long               | \[-3276,8 … 3276,7\] s                                                                                                                                                                                         | k=0,1    |
| 8.004      | DPT_DeltaTime100Msec                     | 2 bytes | Long               | \[-327,68 … 327,67\] s                                                                                                                                                                                         | k=0,01   |
| 8.005      | DPT_DeltaTimeSec                         | 2 bytes | Long               | \[-32 768 … 32 767\] s                                                                                                                                                                                         |          |
| 8.006      | DPT_DeltaTimeMin                         | 2 bytes | Long               | \[-32 768 … 32 767\] min                                                                                                                                                                                       |          |
| 8.007      | DPT_DeltaTimeHrs                         | 2 bytes | Long               | \[-32 768 … 32 767\] h                                                                                                                                                                                         |          |
| 8.010      | DPT\_Percent\_V16                        | 2 bytes | Long               | \[-327,68 … 327,67\] %                                                                                                                                                                                         | k=0,01   |
| 8.011      | DPT\_Rotation\_Angle                     | 2 bytes | Long               | \[-32 768 … 32 767\] °                                                                                                                                                                                         |          |
| 9.001      | DPT\_Value\_Temp                         | 2 bytes | Double             | \[-273 … 670760\] °C                                                                                                                                                                                           |          |
| 9.002      | DPT\_Value\_Tempd                        | 2 bytes | Double             | \[-670760 … 670760\] K                                                                                                                                                                                         |          |
| 9.003      | DPT\_Value\_Tempa                        | 2 bytes | Double             | \[-670760 … 670760\] K/h                                                                                                                                                                                       |          |
| 9.004      | DPT\_Value\_Lux                          | 2 bytes | Double             | \[0 … 670760\] Lux                                                                                                                                                                                             |          |
| 9.005      | DPT\_Value\_Wsp                          | 2 bytes | Double             | \[0 … 670760\] m/s                                                                                                                                                                                             |          |
| 9.006      | DPT\_Value\_Pres                         | 2 bytes | Double             | \[0 … 670760\] Pa                                                                                                                                                                                              |          |
| 9.007      | DPT\_Value\_Humidity                     | 2 bytes | Double             | \[0 … 670760\] %                                                                                                                                                                                               |          |
| 9.008      | DPT\_Value\_AirQuality                   | 2 bytes | Double             | \[0 … 670760\] ppm                                                                                                                                                                                             |          |
| 9.010      | DPT\_Value\_Time1                        | 2 bytes | Double             | \[-670760 … 670760\] s                                                                                                                                                                                         |          |
| 9.011      | DPT\_Value\_Time2                        | 2 bytes | Double             | \[-670760 … 670760\] ms                                                                                                                                                                                        |          |
| 9.020      | DPT\_Value\_Volt                         | 2 bytes | Double             | \[-670760 … 670760\] mV                                                                                                                                                                                        |          |
| 9.021      | DPT\_Value\_Curr                         | 2 bytes | Double             | \[-670760 … 670760\] mA                                                                                                                                                                                        |          |
| 9.022      | DPT_PowerDensity                         | 2 bytes | Double             | \[-670760 … 670760\] W/m2                                                                                                                                                                                      |          |
| 9.023      | DPT_KelvinPerPercent                     | 2 bytes | Double             | \[-670760 … 670760\] K/%                                                                                                                                                                                       |          |
| 9.024      | DPT_Power                                | 2 bytes | Double             | \[-670760 … 670760\] kW                                                                                                                                                                                        |          |
| 9.025      | DPT\_Value\_Volume_Flow                  | 2 bytes | Double             | \[-670760 … 670760\] l/h                                                                                                                                                                                       |          |
| 9.026      | DPT\_Rain\_Amount                        | 2 bytes | Double             | \[-670760 … 670760\] l/m2                                                                                                                                                                                      |          |
| 9.027      | DPT\_Value\_Temp_F                       | 2 bytes | Double             | \[-459,6 … 670760\] °F                                                                                                                                                                                         |          |
| 9.028      | DPT\_Value\_Wsp_kmh                      | 2 bytes | Double             | \[0 … 670760\]km/h                                                                                                                                                                                             |          |
| 10.001     | DPT_TimeOfDay                            | 3 bytes | Long               | D₃H₅R₁R₁M₆rrS₆<br><br>D = \[0 … 7\]; H = \[0 … 23\]; M = \[0 … 59\]; S = \[0 … 59\]; R = {0}                                                                                                                   |          |
| 11.001     | DPT_Date                                 | 3 bytes | Long               | R₁R₁R₁D₅R₁R₁R₁R₁M₄R₁Y₇<br><br>D = \[1 … 31\]; M = \[1 … 12\]; Y = \[0 … 99\]; R = {0}                                                                                                                          |          |
| 12.001     | DPT\_Value\_4_Ucount                     | 4 bytes | Long               | \[0 … 4 294 967 295\]                                                                                                                                                                                          |          |
| 13.001     | DPT\_Value\_4_Count                      | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\]                                                                                                                                                                             |          |
| 13.002     | DPT\_FlowRate\_m3/h                      | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\]                                                                                                                                                                             |          |
| 13.010     | DPT_ActiveEnergy                         | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\] Wh                                                                                                                                                                          |          |
| 13.012     | DPT_ReactiveEnergy                       | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\] VAh                                                                                                                                                                         |          |
| 13.013     | DPT\_ActiveEnergy\_kWh                   | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\] kWh                                                                                                                                                                         |          |
| 13.014     | DPT\_ApparantEnergy\_kVAh                | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\] kVAh                                                                                                                                                                        |          |
| 13.015     | DPT\_ReactiveEnergy\_kVARh               | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\] kVARh                                                                                                                                                                       |          |
| 13.100     | DPT_LongDeltaTimeSec                     | 4 bytes | Long               | \[-2 147 483 648 … 2 147 483 647\] s                                                                                                                                                                           |          |
| 14.000     | DPT\_Value\_Acceleration                 | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.001     | DPT\_Value\_Acceleration_Angular         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.002     | DPT\_Value\_Activation_Energy            | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.003     | DPT\_Value\_Activity                     | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.004     | DPT\_Value\_Mol                          | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.005     | DPT\_Value\_Amplitude                    | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.006     | DPT\_Value\_AngleRad                     | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.007     | DPT\_Value\_AngleDeg                     | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.008     | DPT\_Value\_Angular_Momentum             | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.009     | DPT\_Value\_Angular_Velocity             | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.010     | DPT\_Value\_Area                         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.011     | DPT\_Value\_Capacitance                  | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.012     | DPT\_Value\_Charge_DensitySurface        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.013     | DPT\_Value\_Charge_DensityVolume         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.014     | DPT\_Value\_Compressibility              | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.015     | DPT\_Value\_Conductance                  | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.016     | DPT\_Value\_Electrical_Conductivity      | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.017     | DPT\_Value\_Density                      | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.018     | DPT\_Value\_Electric_Charge              | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.019     | DPT\_Value\_Electric_Current             | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.020     | DPT\_Value\_Electric_CurrentDensity      | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.021     | DPT\_Value\_Electric_DipoleMoment        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.022     | DPT\_Value\_Electric_Displacement        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.023     | DPT\_Value\_Electric_FieldStrength       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.024     | DPT\_Value\_Electric_Flux                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.025     | DPT\_Value\_Electric_FluxDensity         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.026     | DPT\_Value\_Electric_Polarization        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.027     | DPT\_Value\_Electric_Potential           | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.028     | DPT\_Value\_Electric_PotentialDifference | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.029     | DPT\_Value\_ElectromagneticMoment        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.030     | DPT\_Value\_Electromotive_Force          | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.031     | DPT\_Value\_Energy                       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.032     | DPT\_Value\_Force                        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.033     | DPT\_Value\_Frequency                    | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.034     | DPT\_Value\_Angular_Frequency            | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.035     | DPT\_Value\_Heat_Capacity                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.036     | DPT\_Value\_Heat_FlowRate                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.037     | DPT\_Value\_Heat_Quantity                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.038     | DPT\_Value\_Impedance                    | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.039     | DPT\_Value\_Length                       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.040     | DPT\_Value\_Light_Quantity               | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.041     | DPT\_Value\_Luminance                    | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.042     | DPT\_Value\_Luminous_Flux                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.043     | DPT\_Value\_Luminous_Intensity           | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.044     | DPT\_Value\_Magnetic_FieldStrength       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.045     | DPT\_Value\_Magnetic_Flux                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.046     | DPT\_Value\_Magnetic_FluxDensity         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.047     | DPT\_Value\_Magnetic_Moment              | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.048     | DPT\_Value\_Magnetic_Polarization        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.049     | DPT\_Value\_Magnetization                | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.050     | DPT\_Value\_MagnetomotiveForce           | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.051     | DPT\_Value\_Mass                         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.052     | DPT\_Value\_MassFlux                     | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.053     | DPT\_Value\_Momentum                     | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.054     | DPT\_Value\_Phase_AngleRad               | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.055     | DPT\_Value\_Phase_AngleDeg               | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.056     | DPT\_Value\_Power                        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.057     | DPT\_Value\_Power_Factor                 | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.058     | DPT\_Value\_Pressure                     | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.059     | DPT\_Value\_Reactance                    | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.060     | DPT\_Value\_Resistance                   | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.061     | DPT\_Value\_Resistivity                  | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.062     | DPT\_Value\_SelfInductance               | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.063     | DPT\_Value\_SolidAngle                   | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.064     | DPT\_Value\_Sound_Intensity              | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.065     | DPT\_Value\_Speed                        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.066     | DPT\_Value\_Stress                       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.067     | DPT\_Value\_Surface_Tension              | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.068     | DPT\_Value\_Common_Temperature           | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.069     | DPT\_Value\_Absolute_Temperature         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.070     | DPT\_Value\_TemperatureDifference        | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.071     | DPT\_Value\_Thermal_Capacity             | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.072     | DPT\_Value\_Thermal_Conductivity         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.073     | DPT\_Value\_ThermoelectricPower          | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.074     | DPT\_Value\_Time                         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.075     | DPT\_Value\_Torque                       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.076     | DPT\_Value\_Volume                       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.077     | DPT\_Value\_Volume_Flux                  | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.078     | DPT\_Value\_Weight                       | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 14.079     | DPT\_Value\_Work                         | 4 bytes | Double             | Z₁E₈F₂₃<br><br>Z = {0,1}; E = \[0 … 255\]; F = \[0 … 8 388 607\]                                                                                                                                               |          |
| 15.000     | DPT\_Access\_Data                        | 4 bytes | Long               | U₄V₄W₄X₄Y₄Z₄E₁P₁D₁C₁N₄<br><br>U,V,W,X,Y,Z = \[0 … 9\]; E,P,D,C = {0,1}; N = \[0 … 15\]                                                                                                                         |          |
| 17.001     | DPT_SceneNumber                          | 1 byte  | Long               | R₁R₁U₆<br><br>R = {0}; U = \[0 … 63\]                                                                                                                                                                          |          |
| 18.001     | DPT_SceneControl                         | 1 byte  | Long               | C₁R₁U₆<br><br>C = {0,1}; R = {0}; U = \[0 … 63\]                                                                                                                                                               |          |
| 19.001     | DPT_DateTime                             | 8 bytes | String             | U₈R₁R₁R₁R₁M₄R₁R₁R₁D₅W₃H₅R₁R₁S₆R₁R₁S₆E₁F₁G₁I₁J₁K₁L₁N₁O₁R₁R₁R₁R₁R₁R₁R₁<br><br>U = \[0 … 255\]; M = \[1 … 12\]; D = \[1 … 31\]; W = \[0 … 7\]; H = \[0 … 24\]; S = \[0 … 59\]; R = {0}; E,F,G,I,J,K,L,N,O = {0,1} |          |
| 20.001     | DPT_SCLOMode                             | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.002     | DPT_BuildingMode                         | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.003     | DPT_OccMode                              | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.004     | DPT_Priority                             | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.005     | DPT_LightApplicationMode                 | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.006     | DPT_ApplicationArea                      | 1 byte  | Long               | {0,1,10,11,12,13,14}                                                                                                                                                                                           |          |
| 20.007     | DPT_AlarmClassType                       | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.008     | DPT_PSUMode                              | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.011     | DPT\_ErrorClass\_System                  | 1 byte  | Long               | \[0 … 18\]                                                                                                                                                                                                     |          |
| 20.012     | DPT\_ErrorClass\_HVAC                    | 1 byte  | Long               | \[0 … 4\]                                                                                                                                                                                                      |          |
| 20.013     | DPT\_Time\_Delay                         | 1 byte  | Long               | \[0 … 25\]                                                                                                                                                                                                     |          |
| 20.014     | DPT\_Beaufort\_Wind\_Force\_Scale        | 1 byte  | Long               | \[0 … 12\]                                                                                                                                                                                                     |          |
| 20.017     | DPT_SensorSelect                         | 1 byte  | Long               | \[0 … 4\]                                                                                                                                                                                                      |          |
| 20.020     | DPT_ActuatorConnectType                  | 1 byte  | Long               | \[1 … 2\]                                                                                                                                                                                                      |          |
| 20.100     | DPT_FuelType                             | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.101     | DPT_BurnerType                           | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.102     | DPT_HVACMode                             | 1 byte  | Long               | \[0 … 4\]                                                                                                                                                                                                      |          |
| 20.103     | DPT_DHWMode                              | 1 byte  | Long               | \[0 … 4\]                                                                                                                                                                                                      |          |
| 20.104     | DPT_LoadPriority                         | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.105     | DPT_HVACContrMode                        | 1 byte  | Long               | {\[0 … 17\],20}                                                                                                                                                                                                |          |
| 20.106     | DPT_HVACEmergMode                        | 1 byte  | Long               | \[0 … 5\]                                                                                                                                                                                                      |          |
| 20.107     | DPT_ChangeoverMode                       | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.108     | DPT_ValveMode                            | 1 byte  | Long               | \[1 … 5\]                                                                                                                                                                                                      |          |
| 20.109     | DPT_DamperMode                           | 1 byte  | Long               | \[1 … 4\]                                                                                                                                                                                                      |          |
| 20.110     | DPT_HeaterMode                           | 1 byte  | Long               | \[1 … 3\]                                                                                                                                                                                                      |          |
| 20.111     | DPT_FanMode                              | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.112     | DPT_MasterSlaveMode                      | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.113     | DPT_StatusRoomSetp                       | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.120     | DPT_ADAType                              | 1 byte  | Long               | \[1 … 2\]                                                                                                                                                                                                      |          |
| 20.121     | DPT_BackupMode                           | 1 byte  | Long               | \[0 … 1\]                                                                                                                                                                                                      |          |
| 20.122     | DPT_StartSynchronization                 | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.600     | DPT\_Behaviour\_Lock_Unlock              | 1 byte  | Long               | \[0 … 6\]                                                                                                                                                                                                      |          |
| 20.601     | DPT\_Behaviour\_Bus\_Power\_Up_Down      | 1 byte  | Long               | \[0 … 4\]                                                                                                                                                                                                      |          |
| 20.602     | DPT\_DALI\_Fade_Time                     | 1 byte  | Long               | \[0 … 15\]                                                                                                                                                                                                     |          |
| 20.603     | DPT_BlinkingMode                         | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.604     | DPT_LightControlMode                     | 1 byte  | Long               | \[0 … 1\]                                                                                                                                                                                                      |          |
| 20.605     | DPT_SwitchPBModel                        | 1 byte  | Long               | \[1 … 2\]                                                                                                                                                                                                      |          |
| 20.606     | DPT_PBAction                             | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.607     | DPT_DimmPBModel                          | 1 byte  | Long               | \[1 … 4\]                                                                                                                                                                                                      |          |
| 20.608     | DPT_SwitchOnMode                         | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.609     | DPT_LoadTypeSet                          | 1 byte  | Long               | \[0 … 2\]                                                                                                                                                                                                      |          |
| 20.610     | DPT_LoadTypeDetected                     | 1 byte  | Long               | \[0 … 3\]                                                                                                                                                                                                      |          |
| 20.801     | 1 DPT_SABExceptBehaviour                 | 1 byte  | Long               | \[0 … 4\]                                                                                                                                                                                                      |          |
| 20.802     | DPT\_SABBehaviour\_Lock_Unlock           | 1 byte  | Long               | \[0 … 6\]                                                                                                                                                                                                      |          |
| 20.803     | DPT_SSSBMode                             | 1 byte  | Long               | \[1 … 4\]                                                                                                                                                                                                      |          |
| 20.804     | DPT_BlindsControlMode                    | 1 byte  | Long               | \[0 … 1\]                                                                                                                                                                                                      |          |
| 20.1000    | DPT_CommMode                             | 1 byte  | Long               | \[0 … 255\]                                                                                                                                                                                                    |          |
| 20.1001    | DPT_AddInfoTypes                         | 1 byte  | Long               | {00h,01h,02h,03h,04h,05h,06h,07h,FFh}                                                                                                                                                                          |          |
| 20.1002    | DPT\_RF\_ModeSelect                      | 1 byte  | Long               | {00h,01h,02h}                                                                                                                                                                                                  |          |
| 20.1003    | DPT\_RF\_FilterSelect                    | 1 byte  | Long               | {00h,01h,02h,03h}                                                                                                                                                                                              |          |
| 21.001     | DPT_StatusGen                            | 1 byte  | Long               | B₃F₁E₁D₁C₁A₁<br><br>A,C,D,E,F = {0,1}; B = {000}                                                                                                                                                               |          |
| 21.002     | DPT\_Device\_Control                     | 1 byte  | Long               | B₃F₁E₁D₁C₁A₁<br><br>A,C,D,E,F = {0,1}; B = {000}                                                                                                                                                               |          |
| 21.100     | DPT_ForceSign                            | 1 byte  | Long               | H₁G₁F₁E₁D₁C₁B₁A₁<br><br>A,B,C,D,E,F,G,H = {0,1}                                                                                                                                                                |          |
| 21.101     | DPT_ForceSignCool                        | 1 byte  | Long               | B₁A₇<br><br>B = {0,1}; A = {0000000}                                                                                                                                                                           |          |
| 21.102     | DPT_StatusRHC                            | 1 byte  | Long               | H₁G₁F₁E₁D₁C₁B₁A₁<br><br>A,B,C,D,E,F,G,H = {0,1}                                                                                                                                                                |          |
| 21.103     | DPT_StatusSDHWC                          | 1 byte  | Long               | D₅C₁B₁A₁<br><br>A,B,C = {0,1}; D = {00000}                                                                                                                                                                     |          |
| 21.104     | DPT_FuelTypeSet                          | 1 byte  | Long               | D₅C₁B₁A₁<br><br>A,B,C = {0,1}; D = {0}                                                                                                                                                                         |          |
| 21.105     | DPT_StatusRCC                            | 1 byte  | Long               | A₇B₁<br><br>B = {0,1}; A = {0000000}                                                                                                                                                                           |          |
| 21.106     | DPT_StatusAHU                            | 1 byte  | Long               | E₄D₁C₁B₁A₁<br><br>A,B,C,D = {0,1}, E = {0000}                                                                                                                                                                  |          |
| 21.601     | DPT_LightActuatorErrorInfo               | 1 byte  | Long               | H₁G₁F₁E₁D₁C₁B₁A₁<br><br>A,B,C,D,E,F,G = {0,1}, h = {0}                                                                                                                                                         |          |
| 21.1000    | DPT\_RF\_ModeInfo                        | 1 byte  | Long               | D₅C₁B₁A₁<br><br>C,B,A = {0,1}, D = {00000}                                                                                                                                                                     |          |
| 21.1001    | DPT\_RF\_FilterInfo                      | 1 byte  | Long               | D₅C₁B₁A₁<br><br>C,B,A = {0,1}                                                                                                                                                                                  |          |
| 21.1010    | DPT\_Channel\_Activation_8               | 1 byte  | Long               | H₁G₁F₁E₁D₁C₁B₁A₁<br><br>H,G,F,E,D,C,B,A = {0,1}                                                                                                                                                                |          |
| 22.100     | DPT_StatusDHWC                           | 2 bytes | Long               | J₈HGFEDCBA<br><br>J = {00000000}; H,G,F,E,D,C,B,A = {0,1}                                                                                                                                                      |          |
| 22.101     | DPT_StatusRHCC                           | 2 bytes | Long               | R₁O₁N₁M₁L₁K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>R = {0}; O,N,M,L,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                                                                         |          |
| 22.1000    | DPT_Media                                | 2 bytes | Long               | E₁₀D₁C₁F₁B₁A₁F₁<br><br>E = {0000000000}; D,C,B,A, = {0,1}; F = {0}                                                                                                                                             |          |
| 22.1010    | DPT\_Channel\_Activation_16              | 2 bytes | Long               | P₁O₁N₁M₁L₁K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>P,O,N,M,L,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                                                                                |          |
| 23.001     | DPT_OnOffAction                          | 2 bits  | Long               | {00(off), 01(on), 10(off/on), 11(on/off)}                                                                                                                                                                      |          |
| 23.002     | DPT\_Alarm\_Reaction                     | 2 bits  | Long               | {00 (no alarm is used), 01 (alarm position is UP), 10 (alarm position is DOWN)}                                                                                                                                |          |
| 23.003     | DPT\_UpDown\_Action                      | 2 bits  | Long               | {00 (Up), 01 (Down), 01 (UpDown), 11 (DownUp)}                                                                                                                                                                 |          |
| 23.102     | DPT\_HVAC\_PB_Action                     | 2 bits  | Long               | {00 (Comfort/Economy), 01 (Comfort/Nothing), 10 (Economy/Nothing), 11 (Building prot/Auto)}                                                                                                                    |          |
| 25.1000    | DPT_DoubleNibble                         | 1 byte  | Long               | B₄N₄<br><br>B = \[0 … 3\], N = \[0 … 3\]                                                                                                                                                                       |          |
| 26.001     | SceneInfo                                | 1 byte  | Long               | B₁A₂U₆<br><br>B = {0}; A = {0,1}; U = \[0 … 63\]                                                                                                                                                               |          |
| 27.001     | CombinedInfoOnOff                        | 4 bytes | Long               | Σ₁Π₁Λ₁Θ₁Δ₁Γ₁Z₁Y₁X₁W₁V₁U₁T₁S₁R₁Q₁P₁O₁N₁M₁L₁K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>Σ,Π,Λ,Θ,Δ,Γ,Z,Y,X,W,V,U,T,S,R,Q,P,O,N,M,L,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                |          |
| 29.010     | ActiveEnergy_V64                         | 8 bytes | String             | \[-9 223 372 036 854 775 808 … 9 223 372 036 854 775 807\]                                                                                                                                                     |          |
| 29.011     | ApparantEnergy_V64                       | 8 bytes | String             | \[-9 223 372 036 854 775 808 … 9 223 372 036 854 775 807\]                                                                                                                                                     |          |
| 29.012     | ReactiveEnergy_V64                       | 8 bytes | String             | \[-9 223 372 036 854 775 808 … 9 223 372 036 854 775 807\]                                                                                                                                                     |          |
| 30.1010    | Channel\_Activation\_24                  | 3 bytes | Long               | X₁W₁V₁U₁T₁S₁R₁Q₁P₁O₁N₁M₁L₁K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>X,W,V,U,T,S,R,Q,P,O,N,M,L,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                                                |          |
| 31.101     | PB\_Action\_HVAC_Extended                | 3 bits  | Long               | C₁B₁A₁<br><br>C,B,A = {0,1}                                                                                                                                                                                    |          |
| 200.100    | Heat/Cool_Z                              | 2 bytes | Long               | M₇G₁F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E,G = {0,1}; F = {000}; M = {0000000}                                                                                                                                          |          |
| 200.101    | BinaryValue_Z                            | 2 bytes | Long               | M₇G₁F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E,G = {0,1}; F = {000}; M = {0000000}                                                                                                                                          |          |
| 201.100    | HVACMode_Z                               | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 4\]                                                                                                                                              |          |
| 201.102    | DHWMode_Z                                | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 4\]                                                                                                                                              |          |
| 201.104    | HVACContrMode_Z                          | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 20\]                                                                                                                                             |          |
| 201.105    | EnableH/Cstage_Z                         | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 3\]                                                                                                                                              |          |
| 201.107    | BuildingMode_Z                           | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 2\]                                                                                                                                              |          |
| 201.108    | OccMode_Z                                | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 2\]                                                                                                                                              |          |
| 201.109    | HVACEmergMode_Z                          | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 5\]                                                                                                                                              |          |
| 202.001    | RelValue_Z                               | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 255\]                                                                                                                                            |          |
| 202.002    | UCountValue8_Z                           | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 255\]                                                                                                                                            |          |
| 203.002    | TimePeriodMsec_Z                         | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]ms                                                                                                                                      |          |
| 203.003    | TimePeriod10Msec_Z                       | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]ms                                                                                                                                      | k = 0,1  |
| 203.004    | TimePeriod100Msec_Z                      | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]ms                                                                                                                                      | k = 0,01 |
| 203.005    | TimePeriodSec_Z                          | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]s                                                                                                                                       |          |
| 203.006    | TimePeriodMin_Z                          | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]min                                                                                                                                     |          |
| 203.007    | TimePeriodHrs_Z                          | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]h                                                                                                                                       |          |
| 203.011    | UFlowRateLiter/h_Z                       | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]l/h                                                                                                                                     | k = 0,01 |
| 203.012    | UCountValue16_Z                          | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]                                                                                                                                        |          |
| 203.013    | UElCurrentμA_Z                           | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]μA                                                                                                                                      | k = 0,01 |
| 203.014    | PowerKW_Z                                | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]kW                                                                                                                                      |          |
| 203.015    | AtmPressureAbs_Z                         | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 1200\]kbar                                                                                                                                      | k = 0,05 |
| 203.017    | PercentU16_Z                             | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]%                                                                                                                                       | k = 0,01 |
| 203.100    | HVACAirQual_Z                            | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 65 535\]ppm                                                                                                                                     |          |
| 203.101    | WindSpeed_Z                              | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 200\]km/h                                                                                                                                       | k = 0,01 |
| 203.102    | SunIntensity_Z                           | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 1400\]W/m²                                                                                                                                      | k = 0,05 |
| 203.104    | HVACAirFlowAbs_Z                         | 3 bytes | Long               | N₁₆F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[0 … 1400\]m³/h                                                                                                                                      |          |
| 204.001    | RelSignedValue_Z                         | 2 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-100 … 100\]%                                                                                                                                        |          |
| 205.002    | DeltaTimeMsec_Z                          | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]ms                                                                                                                                 |          |
| 205.003    | DeltaTime10Msec_Z                        | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]ms                                                                                                                                 | k = 0,1  |
| 205.004    | DeltaTime100Msec_Z                       | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]ms                                                                                                                                 | k = 0,01 |
| 205.005    | DeltaTimeSec_Z                           | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]s                                                                                                                                  |          |
| 205.006    | DeltaTimeMin_Z                           | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]min                                                                                                                                |          |
| 205.007    | DeltaTimeHrs_Z                           | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]h                                                                                                                                  |          |
| 205.017    | Percent\_V16\_Z                          | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-32 768 … 32 767\]%                                                                                                                                  | k = 0,01 |
| 205.100    | TempHVACAbs_Z                            | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000}; N = \[-273 … 655,34\]°C                                                                                                                                    | k = 0,02 |
| 205.101    | TempHVACRel_Z                            | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000};N = \[-273 … 655,34\]K                                                                                                                                      | k = 0,02 |
| 205.102    | HVACAirFlowRel_Z                         | 3 bytes | Long               | N₈F₃E₁D₁C₁B₁A₁<br><br>A,B,C,D,E = {0,1}; F = {000};N = \[-32 768 … 32 767\]K                                                                                                                                   |          |
| 206.100    | HVACModeNext                             | 3 bytes | Long               | U₁₆N₈<br><br>U = \[0 … 65535\], N=\[0 … 4\]                                                                                                                                                                    |          |
| 206.102    | DHWModeNext                              | 3 bytes | Long               | U₁₆N₈<br><br>U = \[0 … 65535\]; N=\[0 … 4\]                                                                                                                                                                    |          |
| 206.104    | OccModeNext                              | 3 bytes | Long               | U₁₆N₈<br><br>U = \[0 … 65535\]; N=\[0 … 2\]                                                                                                                                                                    |          |
| 206.105    | BuildingModeNext                         | 3 bytes | Long               | U₁₆N₈<br><br>U = \[0 … 65535\]; N=\[0 … 2\]                                                                                                                                                                    |          |
| 207.100    | StatusBUC                                | 2 bytes | Long               | U₈G₂F₁E₁D₁C₁B₁A₁<br><br>U = \[0 … 100\]%; G = {00}; F,E,D,C,B,A = {0,1}                                                                                                                                        |          |
| 207.101    | LockSign                                 | 2 bytes | Long               | U₈C₆B₁A₁<br><br>U = \[0 … 100\]%; c = {0}; b,a = {0,1}                                                                                                                                                         |          |
| 207.102    | ValueDemBOC                              | 2 bytes | Long               | U₈C₆B₁A₁<br><br>U = \[0 … 100\]%; C = {000000}; B,A = {0,1}                                                                                                                                                    |          |
| 207.104    | ActPosDemAbs                             | 2 bytes | Long               | U₈E₄D₁C₁B₁A₁<br><br>U = \[0 … 100\]%; E = {0000}; D,C,B,A = {0,1}                                                                                                                                              |          |
| 207.105    | StatusAct                                | 2 bytes | Long               | U₈;E₄D₁C₁B₁A₁<br><br>U = \[0 … 100\]%; E = {0000}; D,C,B,A = {0,1}                                                                                                                                             |          |
| 207.600    | StatusLightingActuator                   | 2 bytes | Long               | U₈H₁G₁F₁E₁D₁C₁B₁A₁<br><br>U = \[0 … 100\]%; H,G,F,E,D,C,B,A = {0,1}                                                                                                                                            |          |
| 209.100    | StatusHPM                                | 3 bytes | Long               | V₁₆F₃E₁D₁C₁B₁A₁<br><br>V=\[-273 … 655,34\]°C; F = {000}; E,D,C,B,A = {0,1}                                                                                                                                     | k = 0,02 |
| 209.101    | TempRoomDemAbs                           | 3 bytes | Long               | V₁₆E₄D₁C₁B₁A₁<br><br>V=\[-273 … 655,34\]°C; E = {0000}; D,C,B,A = {0,1}                                                                                                                                        | k = 0,02 |
| 209.102    | StatusCPM                                | 3 bytes | Long               | V₁₆E₄D₁C₁B₁A₁<br><br>V=\[-273..655,34\]°C; E = {0000}; D,C,B,A = {0,1}                                                                                                                                         | k = 0,02 |
| 209.103    | StatusWTC                                | 3 bytes | Long               | V₁₆D₅C₁B₁A₁<br><br>V=\[-273 … 655,34\]°C; D = {00000}; C,B,A = {0,1}                                                                                                                                           | k = 0,02 |
| 210.100    | TempFlowWaterDemAbs                      | 4 bytes | Long               | V₁₆M₄L₁K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>V=\[-273 … 655,34\]°C; M = {0000}; L,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                                                        | k = 0,02 |
| 211.100    | EnergyDemWater                           | 2 bytes | Long               | U₈V₈<br><br>U = \[0 … 100\]; N = \[0 … 20\]                                                                                                                                                                    |          |
| 212.100    | TempRoomSetpSetShift\[3\]                | 6 bytes | String             | A₁₆B₁₆C₁₆<br><br>A,B,C = \[-655,34 … 655,34\]K                                                                                                                                                                 | k = 0,02 |
| 212.101    | TempRoomSetpSet\[3\]                     | 6 bytes | String             | A₁₆B₁₆C₁₆<br><br>A,B,C = \[-655,34 … 655,34\]°C                                                                                                                                                                | k = 0,02 |
| 213.100    | TempRoomSetpSet\[4\]                     | 8 bytes | String             | A₁₆B₁₆C₁₆D₁₆<br><br>A,B,C,D = \[-655,34 … 655,34\]°C                                                                                                                                                           | k = 0,02 |
| 213.101    | TempDHWSetpSet\[4\]                      | 8 bytes | String             | A₁₆B₁₆C₁₆D₁₆<br><br>A,B,C,D = \[-655,34 … 655,34\]°C                                                                                                                                                           | k = 0,02 |
| 213.102    | TempRoomSetpSetShift\[4\]                | 8 bytes | String             | A₁₆B₁₆C₁₆D₁₆<br><br>A,B,C,D = \[-655,34 … 655,34\]K                                                                                                                                                            | k = 0,02 |
| 214.100    | PowerFlowWaterDemHPM                     | 4 bytes | Long               | V₁₆U₈G₂F₁E₁D₁C₁B₁A₁<br><br>V = \[-273 … 655,34\]°C; U = \[0 … 100\]%; H = {00}; F,E,D,C,B,A = {0,1}                                                                                                            | k = 0,02 |
| 214.101    | PowerFlowWaterDemCPM                     | 4 bytes | Long               | V₁₆U₈H₅C₁B₁A₁<br><br>V = \[-273 … 655,34\]°C; U = \[0 … 100\]%; D = {00000}; C,B,A = {0,1}                                                                                                                     | k = 0,02 |
| 215.100    | StatusBOC                                | 5 bytes | String             | V₁₆U₈M₄L₁K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>V = \[-273 … 655,34\]°C; U = \[0 … 100\]%; M = {0000}; L,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                                  | k = 0,02 |
| 215.101    | StatusCC                                 | 5 bytes | String             | V₁₆U₈J₈H₁G₁F₁E₁D₁C₁B₁A₁<br><br>V = \[-273 … 655,34\]°C; U = \[0 … 100\]%; J = {00000000}; H,G,F,E,D,C,B,A = {0,1}                                                                                              |          |
| 216.100    | SpecHeatProd                             | 5 bytes | String             | V₁₆U₈N₈D₅C₁B₁A₁<br><br>V = \[0 … 65 535\]kW; U = \[0 … 100\]%; N = \[0..3\]; D = {00000}; C,B,A = {0,1}                                                                                                        |          |
| 217.001    | Version                                  | 2 bytes | Long               | U₅V₅W₆<br><br>U = \[0 … 31\], V = \[0..31\], W = \[0 … 63\]                                                                                                                                                    |          |
| 218.001    | VolumeLiter_Z                            | 5 bytes | String             | V₃₂F₃E₁D₁C₁B₁A₁<br><br>E,D,C,B,A = {0,1}; F = {000}; V = \[-2 147 483 648 l … 2 147 483 647\]l                                                                                                                 |          |
| 218.002    | FlowRate\_m3/h\_Z                        | 5 bytes | String             | V₃₂F₃E₁D₁C₁B₁A₁<br><br>E,D,C,B,A = {0,1}; F = {000}; V = \[-2 147 483 648 l … 2 147 483 647\]m³/h                                                                                                              |          |
| 219.001    | AlarmInfo                                | 6 bytes | String             | U₈N₈V₈W₈P₂N₁M₁L₁K₁J₁I₁D₅C₁B₁A₁<br><br>U = \[0 … 255\]; N = \[0 … 3\]; V = \[0 … 50\]; W = \[0 … 6\]; P = {00}; D = {00000}; A,B,C,I,J,K,L,M,N = {0,1}                                                          |          |
| 220.100    | TempHVACAbsNext                          | 4 bytes | Long               | U₁₆V₁₆<br><br>U = \[0 … 65535\]; V = \[-273 … 655,34\]                                                                                                                                                         | k = 0,02 |
| 221.001    | SerNum                                   | 6 bytes | String             | N₁₆U₃₂<br><br>U = \[0 … 65535\]; V = \[0 … 4 294 967 295\]                                                                                                                                                     |          |
| 222.100    | TempRoomSetpSetF16\[3\]                  | 6 bytes | String             | G₁₆F₁₆E₁₆<br><br>G,F,E = \[-273 … 670 760\]°C                                                                                                                                                                  |          |
| 222.101    | TempRoomSetpSetShiftF16\[3\]             | 6 bytes | String             | G₁₆F₁₆E₁₆<br><br>G,F,E = \[-670 760 … 670 760\]K                                                                                                                                                               |          |
| 223.100    | EnergyDemAir                             | 3 bytes | Long               | G₈F₈E₈<br><br>G = \[-100 … 100\]; F = \[0 … 20\]; E = \[0 … 5\]                                                                                                                                                |          |
| 224.100    | TempSupplyAirSetpSet                     | 6 bytes | String             | U₁₆V₁₆F₈E₈<br><br>U,V = \[-273 … 655,34\]; F = \[0 … 20\]; E = \[0 … 5\]                                                                                                                                       | k = 0,02 |
| 225.001    | ScalingSpeed                             | 3 bytes | Long               | U₁₆V₈<br><br>U = \[1 … 65 535\]; V = \[0,4 … 100\]                                                                                                                                                             | k = 0,4  |
| 225.002    | Scaling\_Step\_Time                      | 3 bytes | Long               | U₁₆V₈<br><br>U = \[1 … 65 535\]; V = \[0,4 … 100\]                                                                                                                                                             | k = 0,4  |
| 225.003    | TariffNext                               | 3 bytes | Long               | U₁₆V₈<br><br>U = \[0 … 65 535\]; V = \[0 … 254\]                                                                                                                                                               |          |
| 229.001    | MeteringValue                            | 6 bytes | String             | V₃₂N₈F₃E₁D₁C₁B₁A₁<br><br>E,D,C,B,A = {0,1}; f,g,h = {000}; V = \[-2 147 483 648 l … 2 147 483 647\]; N = \[0 … 177\];                                                                                          |          |
| 230.1000   | MBus_Address                             | 8 bytes | String             | U₁₆V₃₂W₈N₈<br><br>"MBus address", parameters U,V,W,N are manufacturer specific                                                                                                                                 |          |
| 231.001    | Locale_ASCII                             | 4 bytes | Long               | L₁₆R₁₆<br><br>L - language; R - region                                                                                                                                                                         |          |
| 232.600    | Colour_RGB                               | 3 bytes | Long               | R₈G₈B₈<br><br>R,G,B = \[0..255\]                                                                                                                                                                               |          |
| 234.001    | LanguageCodeAlpha2                       | 2 bytes | Long               | A₁₆<br><br>A - language code                                                                                                                                                                                   |          |
| 234.002    | RegionCodeAlpha2                         | 2 bytes | Long               | A₁₆<br><br>A - region code                                                                                                                                                                                     |          |
| 235.001    | Tariff_ActiveEnergy                      | 6 bytes | String             | V₃₂U₈C₆B₁A₁<br><br>V = \[-2 147 483 648 l … 2 147 483 647\]; U = \[0 … 254\]; C = {000000}; B,A = {0,1}                                                                                                        |          |
| 236.001    | Prioritised\_Mode\_Control               | 1 byte  | Long               | B₁P₃M₄<br><br>B = {0,1}; P = \[0 … 7\]; M = \[0 … 15\]                                                                                                                                                         |          |
| 237.600    | DALI\_Control\_Gear_Diagnostic           | 2 bytes | Long               | M₅L₁K₁J₁H₁G₁A₆<br><br>M = {00000}; L,K,J,H,G = {0,1}; A = \[0 … 63\]                                                                                                                                           |          |
| 238.001    | SceneConfig                              | 1 byte  | Long               | A₁B₁N₆<br><br>A,B = {0,1}; N = \[0 … 63\]                                                                                                                                                                      |          |
| 238.600    | DALI_Diagnostic                          | 1 byte  | Long               | A₁B₁N₆<br><br>A,B = {0,1}; N = \[0 … 63\]                                                                                                                                                                      |          |
| 239.001    | FlaggedScaling                           | 2 bytes | Long               | U₈B₇A₁<br><br>U = \[0 … 100\]; B = {0000000}; A = {0,1}                                                                                                                                                        | k = 0,4  |
| 240.800    | CombinedPosition                         | 3 bytes | Long               | U₁₆C₆B₁A₁<br><br>U = \[0 … 100\]; C = {000000}; B,A = {0,1}                                                                                                                                                    | k = 0,4  |
| 241.800    | StatusSAB                                | 4 bytes | Long               | U₈V₈N₁M₁L₃K₁J₁I₁H₁G₁F₁E₁D₁C₁B₁A₁<br><br>U,V = \[0 … 100\]; L = {000}; N,M,K,J,I,H,G,F,E,D,C,B,A = {0,1}                                                                                                        | k = 0,4  |